<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class DashboardFactory
{

    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }

    public function ultimasTarefasRealizadas($listaOperadores)
    {

        $sql = "SELECT ULTTAR.CODIGO, ULTTAR.NRLOCALCLIENTE, ULTTAR.NRCHECKLIST, ULTTAR.CDOPERADOR, 
                TO_CHAR(ULTTAR.DTULTATU, 'DD/MM') DTAGEND, TO_CHAR(ULTTAR.DTULTATU, 'HH24:MI') HORAAGEND, ULTTAR.NOME,
                ULTTAR.SITUACAO, ULTTAR.TIPO, FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                LCP.NMLOCALCLIENTE AS NMLOCALPAI
                FROM (SELECT NRGERENCIAMENTOCHK AS CODIGO, NRLOCALCLIENTE, NRCHECKLIST,
                             CDOPERADOR, DTULTATU, 'Vistoria: '||NMGERENCIAMENTOCHK AS NOME , IDSITUACAOGERCHK AS SITUACAO,
                             'V' AS TIPO
                        FROM FCM_GERENCIAMENTOCHK
                       WHERE IDSITUACAOGERCHK = '3'

                       UNION ALL

                       SELECT NRCHECKLISTAGEND AS CODIGO, NRLOCALCLIENTE, NRCHECKLIST,
                              CDOPERADOR, DTULTATU, NMCHECKLISTAGEND AS NOME , IDSITUACAOAGEND AS SITUACAO,
                              'T' AS TIPO
                         FROM FCM_CHECKLISTAGEND
                        WHERE IDSITUACAOAGEND = '4') ULTTAR
                JOIN OPERADOR FUNC
                ON ULTTAR.CDOPERADOR = FUNC.CDOPERADOR
                JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = ULTTAR.NRLOCALCLIENTE
                JOIN FCM_LOCALCLI LCP
                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                WHERE (ULTTAR.CDOPERADOR IN ({$listaOperadores}) OR ULTTAR.CDOPERADOR IS NULL)
               AND ULTTAR.DTULTATU BETWEEN SYSDATE - 7 AND SYSDATE
                ORDER BY DTAGEND DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function dataBaseChartPie($parametros, $listaOperadores)
    {

        $sql = "SELECT AT.NMATIVIDADE, COUNT(CA.NRCHECKLISTAGEND) AS QUANTIDADE, AT.DSCORIDATIVIDADE
                      FROM FCM_CHECKLISTAGEND CA
                      JOIN FCM_ATIVIDADECHK AC
                        ON AC.NRCHECKLIST = CA.NRCHECKLIST
                      JOIN FCM_ATIVIDADE AT
                        ON AT.NRATIVIDADE = AC.NRATIVIDADE
                       WHERE TRUNC(DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)
                       AND IDSITUACAOAGEND = '4'
                        AND (
                        (CA.CDOPERADOR IN ($listaOperadores)) 
                        OR (CA.CDOPERADOR IS NULL AND CA.IDTAREFAEQUIPE = 'N') 
                        OR (CA.CDOPERINCLUSAO = :CDOPERADOR AND CA.IDATIVO = 'S')
                          )
                  GROUP BY AT.NMATIVIDADE, AT.DSCORIDATIVIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function dataBaseChartBar($parametros, $listaOperadores)
    {

        $sql = "SELECT CTGC.NRCRITERIOTAREFACHK, CTGC.NMCRITERIOTARGERCHK, COUNT(CTGC.NRCRITERIOTAREFACHK) AS QTDADE, AT.DSCORIDATIVIDADE
              FROM FCM_CRITERIOTARGERCHK CTGC
              JOIN FCM_TAREFAGERCHK TG
              ON TG.NRTAREFAGERCHK = CTGC.NRTAREFAGERCHK
              JOIN FCM_GERENCIAMENTOCHK GCHK
              ON GCHK.NRGERENCIAMENTOCHK = TG.NRGERENCIAMENTOCHK
              JOIN FCM_ATIVIDADECHK AC
              ON AC.NRCHECKLIST = GCHK.NRCHECKLIST
              JOIN FCM_ATIVIDADE AT
              ON AT.NRATIVIDADE = AC.NRATIVIDADE
              WHERE CTGC.IDMARCADO = 'S'
              AND TRUNC(GCHK.DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)
                  AND (
                  (GCHK.CDOPERADOR IN ($listaOperadores)) 
                  OR (GCHK.CDOPERADOR IS NULL))
                  GROUP BY CTGC.NRCRITERIOTAREFACHK, CTGC.NMCRITERIOTARGERCHK, AT.DSCORIDATIVIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function tarefasAtrasadas($parametros, $listaOperadores)
    {

        $sql = "SELECT NRCHECKLISTAGEND, NRLOCALCLIENTE, NRCHECKLIST, CDOPERADOR, IDSITUACAOAGEND, DTHORAAGEND, NMLOCALCLIENTE, LOCALPAI, NMLOCALPAI,
                 HORAS AS DTHORAATRASO, NMOPERADOR, NMCHECKLISTAGEND
                 FROM (SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NMCHECKLISTAGEND,
                        TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTHORAAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, F.NMOPERADOR,
                        LCP.NMLOCALCLIENTE AS NMLOCALPAI, 
                        TRUNC(SYSDATE - CA.DTHORAAGEND) * 24 + TRUNC(((SYSDATE - CA.DTHORAAGEND) - TRUNC(SYSDATE - CA.DTHORAAGEND)) * 24) AS HORAS,
                        TRUNC(((((SYSDATE - CA.DTHORAAGEND) - TRUNC(SYSDATE - CA.DTHORAAGEND)) * 24) - TRUNC(((SYSDATE - CA.DTHORAAGEND) - TRUNC(SYSDATE - CA.DTHORAAGEND)) * 24)) * 60) AS MINUTOS
                      FROM FCM_CHECKLISTAGEND CA
                      JOIN FCM_LOCALCLI LC
                      ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                      JOIN FCM_LOCALCLI LCP
                      ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                      JOIN FCM_CHECKLIST CHK
                      ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                      JOIN OPERADOR F
                      ON F.CDOPERADOR = CA.CDOPERADOR
                      WHERE ((IDSITUACAOAGEND = '5') OR ((IDSITUACAOAGEND = '1') AND (CA.DTHORAAGEND <= SYSDATE)))
                          AND (
                              (CA.CDOPERADOR IN ($listaOperadores)) 
                              OR (CA.CDOPERADOR IS NULL AND CA.IDTAREFAEQUIPE = 'N') 
                              OR (CA.CDOPERINCLUSAO = :CDOPERADOR AND CA.IDATIVO = 'S')
                          )
                     ) TAREFASATRASADAS";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function tarefasFinalizadas($parametros, $listaOperadores)
    {
        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NMCHECKLISTAGEND,
                TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTAGEND, TO_CHAR(CA.DTHORAAGEND, 'HH24:MI') HORAAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                LCP.NMLOCALCLIENTE AS NMLOCALPAI, TO_CHAR(CA.DTULTATU, 'DD/MM HH24:MI') DTULTATU, GCHK.NRGERENCIAMENTOCHK, GCHK.IDSITUACAOGERCHK, F.NMOPERADOR
                        FROM FCM_CHECKLISTAGEND CA
                        JOIN FCM_LOCALCLI LC
                        ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                        JOIN FCM_LOCALCLI LCP
                        ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                        JOIN FCM_CHECKLIST CHK
                        ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                        JOIN OPERADOR F
                        ON F.CDOPERADOR = CA.CDOPERADOR
                        JOIN FCM_GERENCIAMENTOCHK GCHK
                        ON GCHK.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
                        WHERE CA.IDSITUACAOAGEND = '4'
                        AND COALESCE(GCHK.IDSITUACAOGERCHK, '1') = '1'
                        AND (
                        (CA.CDOPERADOR IN ($listaOperadores)) 
                        OR (CA.CDOPERADOR IS NULL AND CA.IDTAREFAEQUIPE = 'N') 
                        OR (CA.CDOPERINCLUSAO = :CDOPERADOR AND CA.IDATIVO = 'S')
                          )
                        AND TRUNC(CA.DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function tarefasConcluidas($parametros, $listaOperadores)
    {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NMCHECKLISTAGEND,
                  TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTAGEND, TO_CHAR(CA.DTHORAAGEND, 'HH24:MI') HORAAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                  LCP.NMLOCALCLIENTE AS NMLOCALPAI, TO_CHAR(CA.DTULTATU, 'DD/MM HH24:MI')DTULTATU, GCHK.NRGERENCIAMENTOCHK, GCHK.IDSITUACAOGERCHK, F.NMOPERADOR
                        FROM FCM_CHECKLISTAGEND CA
                        JOIN FCM_LOCALCLI LC
                        ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                        JOIN FCM_LOCALCLI LCP
                        ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                        JOIN FCM_CHECKLIST CHK
                        ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                        JOIN OPERADOR F
                        ON F.CDOPERADOR = CA.CDOPERADOR
                        LEFT JOIN FCM_GERENCIAMENTOCHK GCHK
                        ON GCHK.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
                        WHERE CA.IDSITUACAOAGEND = '4'
                        AND (GCHK.IDSITUACAOGERCHK = '3' OR GCHK.NRCHECKLISTAGEND IS NULL)
                        AND (
                        (CA.CDOPERADOR IN ($listaOperadores)) 
                        OR (CA.CDOPERADOR IS NULL AND CA.IDTAREFAEQUIPE = 'N') 
                        OR (CA.CDOPERINCLUSAO = :CDOPERADOR AND CA.IDATIVO = 'S')
                          )";


        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
}
