import { Utilities } from '@/util/Utilities';
import {
    Button, Checkbox, DateRange, FileInput, Form, LoadingService, Modal, ModalService, SelectMultiple, Text, Textarea, TextInput,
} from '@zeedhi/common';
import {
    Config,
    dayjs, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    IterableCarousel, IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import { Loader } from '@googlemaps/js-api-loader';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';
import { TarefasResources } from '@/server/config/Resources';
import { RichText } from '@zeedhi/zd-richtext';
import FileUpload from './FileUpload';
import { FiltroUtil } from './filtroUtil';

export class Fm00001TarefasController {
    public desabilitarFinalizacaoTarefa: boolean = false;

    public desabilitarChk: boolean = false;

    public visibilidadePaginaInicial: boolean = true;

    public visibilidadeDetalhamento: boolean = false;

    public visibilidadeReportar: boolean = false;

    public visibilidadeCarroselImagens: boolean = false;

    public statusTarefa = '';

    public nomeTarefa = '';

    public localTarefa = '';

    public dataTarefa = '';

    public modalFiltrarMinhasTarefas!: Modal;

    public modalAnexarImagem!: Modal;

    public nrlocalcli: any;

    private nrChecklistAgend = '';

    private modoAberturaReportar: boolean = false;

    public telaTarefasOffline!: Modal;

    public desabilitarSalvarObs: any = true;

    public tipoOrdenacao = '';

    public modalModeloPop!: Modal;

    public modalOpcoesAberturaLink!: Modal;

    public async criarModais() {
        this.modalFiltrarMinhasTarefas = await ModalService.createFromJson('fm_00001_000_tarefas/modal/fm_00001_000_filtrar_minhas_tarefas', true, true);
        this.modalAnexarImagem = await ModalService.createFromJson('fm_00001_000_tarefas/modal/fm_00001_000_anexar_imagens', true, true);
        this.modalModeloPop = await ModalService.createFromJson('fm_00001_000_tarefas/modal/fm_modal_modelo_pop', true, true);
        this.modalOpcoesAberturaLink = await ModalService.createFromJson('fm_00001_000_tarefas/modal/fm_modal_opcoes_abertura_link', true, true);
    }

    public destruirModais() {
        ModalService.delete(this.modalFiltrarMinhasTarefas);
        ModalService.delete(this.modalAnexarImagem);
        ModalService.delete(this.modalModeloPop);
        ModalService.delete(this.modalOpcoesAberturaLink);
    }

    public abrirModalFiltrar() {
        this.modalFiltrarMinhasTarefas.show();
    }

    public fecharModalFiltrar() {
        this.modalFiltrarMinhasTarefas.hide();
    }

    public abrirModalAnexar() {
        this.modalAnexarImagem.show();
    }

    public fecharModalAnexar() {
        this.modalAnexarImagem.hide();
    }

    public async abrirModalModeloPop() {
        await this.modalModeloPop.show();
        this.montarInformacoesModeloPop();
    }

    public fecharModalModeloPop() {
        this.modalModeloPop.hide();
    }

    public abrirModalOpcoesAbertura() {
        this.modalOpcoesAberturaLink.show();
    }

    public fecharModalOpcoesAbertura() {
        this.modalOpcoesAberturaLink.hide();
    }

    public ordenarCards({ component }: IEventParam<Button>) {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        iterable.datasource.clearOrder();
        if (component.name === 'botao-ordenar-tarefas') {
            iterable.datasource.addOrder('NMCHECKLISTAGEND', 'ASC');
        } else if (component.name === 'botao-ordenar-local') {
            iterable.datasource.addOrder('NMLOCALCLIENTE', 'ASC');
        } else if (component.name === 'botao-ordenar-data') {
            iterable.datasource.addOrder('DTHORAAGEND', 'DESC');
        } else {
            iterable.datasource.clearOrder();
        }

        const { order } = iterable.datasource;
        if (order) {
            const campo = (order.length === 0) ? '' : order[0].split('.')[0];
            this.exibirTipoOrdenacao(campo);
        }
        iterable.datasource.setPage(1);
        this.inserirOrdenacao({ component: iterable });
    }

    public exibirTipoOrdenacao(name: any) {
        const visibilidadeText = Metadata.getInstance<Text>('titulo-ordenacao');

        switch (name) {
            case 'NMCHECKLISTAGEND':
                this.tipoOrdenacao = 'Ordenado por nome da Tarefa';
                visibilidadeText.isVisible = true;
                break;
            case 'NMLOCALCLIENTE':
                this.tipoOrdenacao = 'Ordenado por Local';
                visibilidadeText.isVisible = true;
                break;
            case 'DTHORAAGEND':
                this.tipoOrdenacao = 'Ordenado por Data';
                visibilidadeText.isVisible = true;
                break;
            default:
                this.tipoOrdenacao = '';
                visibilidadeText.isVisible = false;
                break;
        }
    }

    public inserirOrdenacao({ component }: IEventParam<IterableComponentRender>) {
        const { order } = component.datasource;
        if (order) {
            HttpCRUDTranslator.post('/insertOrdenacao', {
                NMTELA: 'Minhas Tarefas',
                NMCAMPO: (order.length === 0) ? '' : order[0].split('.')[0],
                IDSENTIDO: (order.length === 0) ? '' : order[0].split('.')[1],
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }

    public onMountedOrdenacao({ component }: IEventParam<IterableComponentRender>) {
        const filter = [
            { name: 'NMTELA', operation: '=', value: 'Minhas Tarefas' },
        ];
        HttpCRUDTranslator.get('/getOrdenacao', filter).then((response: IDictionary) => {
            if (response.data.data.length !== 0 && typeof response.data.data.length !== 'undefined') {
                component.datasource.clearOrder();
                component.datasource.addOrder(response.data.data[0].NMCAMPO, response.data.data[0].IDSENTIDO);
                this.exibirTipoOrdenacao(response.data.data[0].NMCAMPO);
                Utilities.showSuccessMessage();
            }
            this.loadTarefas({ component });
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public sortFunction(value1: any, value2: any, columnName: string): boolean {
        if (columnName === 'DTHORAAGEND') {
          return (dayjs(value1, 'DD/MM/YYYY HH24:MI').isAfter(dayjs(value2, 'DD/MM/YYYY HH24:MI')));
        }
        return (value1 > value2);
    }

    public exibirImagemAnexada({ component }: any) {
        const { datasource } = Metadata.getInstance<IterableCarousel>('carousel-imagens').iterable;
        const imagens: any = [];

        if (component.value.length) {
            this.visibilidadeCarroselImagens = true;
            component.value.forEach((elemento: any, posicao: any) => {
                imagens.push({
                    id: posicao,
                    src: window.URL.createObjectURL(elemento),
                });
            });
        } else {
            this.visibilidadeCarroselImagens = false;
        }

        datasource.updateData(imagens);
    }

    public async anexarImagens() {
        const imagensConvertidas = await FileUpload.converterImagens('file-input-anexar-imagem');
        const fileInput = Metadata.getInstance<FileInput>('file-input-anexar-imagem');
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;
        if (fileInput.value) {
            this.rotaAnexarImagens(imagensConvertidas, nr).then(() => {
                Utilities.showSuccessMessage();
                this.fecharModalAnexar();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Nenhuma imagem foi anexada', 'error');
        }
    }

    public async filtrarCards() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        const filtros = {
            CDCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-cliente').value,
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-local').value,
            NRSEQCONTRATO: Metadata.getInstance<SelectMultiple>('filtro-contratos').value,
            NRCHECKLISTAGEND: Metadata.getInstance<SelectMultiple>('filtro-tarefas').value,
            DTHORAGEND: Metadata.getInstance<DateRange>('data').value,
            IDSITUACAOAGEND: Metadata.getInstance<DateRange>('filtro-situacao').value,
        };

        const filtrosFiltrados = FiltroUtil.filtrarFiltros(filtros);

        iterable.datasource.setFilter(filtrosFiltrados);

        this.loadTarefas({ component: iterable });
        this.fecharModalFiltrar();
    }

    public montarFiltros(valoresFiltro: any) {
        const filter = Object.entries(valoresFiltro ?? {})
            .filter(([, value]) => {
                if (Array.isArray(value)) {
                    return value.length > 0;
                }
                return value !== null && value !== undefined;
            })
            .map(([name, value]) => ({
                name,
                operation: '=',
                value: Array.isArray(value) ? value : [value],
            }));

        return filter;
    }

    public limparFiltros() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        Metadata.getInstance<SelectMultiple>('filtro-cliente').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-local').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-contratos').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-tarefas').value = null;
        Metadata.getInstance<DateRange>('data').value = null;

        iterable.datasource.clearFilter();
        this.onMountedOrdenacao({ component: iterable });
        this.fecharModalFiltrar();
    }

    public async activateClickCard(nr: any) {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');
        iterable.datasource.setFilter({
            NRCHECKLISTAGEND: nr,
        });
    }

    public clickCard({ component }: any) {
        this.visibilidadePaginaInicial = false;
        this.visibilidadeDetalhamento = !this.visibilidadePaginaInicial;

        this.montarInformacoesComplementares(component.name.split('_')[1]);
    }

    public retornarPaginaInicial() {
        this.visibilidadePaginaInicial = true;
        this.visibilidadeDetalhamento = !this.visibilidadePaginaInicial;
        this.limparCampos();
    }

    public limparCampos() {
        Metadata.getInstance<Textarea>('box-observation').value = null;
    }

    // Funções da tela de Detalhamento.
    public alterarBotao({ component }: IEventParam<Button>) {
        const status = Metadata.getInstance<TextInput>('idsituacao');
        if (status.value) {
            LoadingService.show();
            HttpCRUDTranslator.get('/qtdTarefasAtivas').then((response: IDictionary) => {
                if (response.data.data[0].TAREFAATIVA >= 0 && status.value !== '2') {
                    this.estilizarBotao({ component }, 'Iniciar');
                    component.disabled = false;
                } else if (status.value === '2') {
                    this.estilizarBotao({ component }, 'Finalizar');
                    component.disabled = false;
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public estilizarBotao({ component }: IEventParam<Button>, tipo: any) {
        if (tipo === 'Iniciar') {
            component.label = 'Iniciar Tarefa';
            component.iconName = 'mdi-chevron-right';
            component.color = 'primary';
        } else {
            component.label = 'Finalizar Tarefa';
            component.iconName = 'mdi-chevron-right';
            component.color = 'success';
        }
    }

    public async iniciarTarefa(nr: any, tipo: string, id: any) {
        let validador = true;
        if (!this.utilizarMirageJs && !validador) {
            this.criaMinhaLocalizacao();
            await this.getParametrização();
            if (this.parametrizacao.IDUTILIZAGEOLOCALINITAR === 'S') {
                validador = await this.googleMaps();
            }
        }
        if (validador) {
            const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

            LoadingService.show();
            this.rotaIniciarTarefa(nr, tipo, id).then(() => {
                Utilities.alertMessage('Tarefa iniciada com sucesso!', 'success');
                iterable.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
            return true;
        }
        Utilities.alertMessage('Você está fora da local para inicio da tarefa.', 'error');
        return false;
    }

    public async iniciarOuFinalizar({ component }: any) {
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;

        if (!(component.label === 'Iniciar Tarefa')) {
            this.statusTarefa = "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
            this.finalizarTarefa(nr);
        } else if (await this.iniciarTarefa(nr, 'iniciar', '2')) {
            this.estilizarBotao({ component }, 'Finalizar');
            component.disabled = false;
            this.statusTarefa = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        }
    }

    public finalizarTarefa(nr: any) {
        const cardsPrincipais = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        if (Utilities.validateForm(Metadata.getInstance<Form>('Form-observation'))) {
            LoadingService.hide();
            this.rotaFinalizarTarefa(nr).then(() => {
                Utilities.showSuccessMessage();
                this.visibilidadeDetalhamento = false;
                this.visibilidadePaginaInicial = true;
                this.limparCampos();
                cardsPrincipais.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public visibleCardsEpis: boolean | undefined = false;

    public async carregarEpis(nrChecklistAgend: any) {
        const iterableEpis = Metadata.getInstance<IterableComponentRender>('iterable-cards-epis');
        await iterableEpis.datasource.setFilter({
            NRCHECKLISTAGEND: parseInt(nrChecklistAgend, 10),
        });
        this.loadEpisTarefas({ component: iterableEpis });
        this.visibleCardsEpis = (iterableEpis.datasource?.data && iterableEpis.datasource?.data.length > 0);
    }

    public visibleCardsModeloPop: boolean | undefined = false;

    private dadosModeloPop: any;

    public clickModeloPop() {
        const link = this.dadosModeloPop[0].LINKMODELOPOP;
        const descricao = this.dadosModeloPop[0].DESCMODELOPOP;
        const observacao = this.dadosModeloPop[0].DSOBSERVACAO;
        const linkArquivo = this.dadosModeloPop[0].DSCAMINHOARQ;

        const hasDescricao = descricao && descricao !== '<p></p>'; // Verifica se existe descrição válida
        const hasObservacao = observacao !== null; // Verifica se existe observação válida

        // Cenário 1: Apenas o link
        if (link && !linkArquivo && !hasDescricao && !hasObservacao) {
           this.abrirLink();
        }

        // Cenário 2: Apenas o linkArquivo
        if (linkArquivo && !link && !hasDescricao && !hasObservacao) {
            this.abrirPdf();
        }

        // Cenário 3: Apenas link e linkArquivo
        if (link && linkArquivo && !hasDescricao && !hasObservacao) {
            this.abrirModalOpcoesAbertura();
        }

        // Cenário 4: Qualquer um dos dois (link ou linkArquivo) + descrição ou observação
        if ((link || linkArquivo) && (hasDescricao || hasObservacao)) {
            this.abrirModalModeloPop(); // Abre o modal
        }
    }

    public abrirPdf() {
        const codigo = this.dadosModeloPop[0].NRANEXOMODELOPOP;

        const downloadUri = `${Config.env.endPoint}downLoadAttachment?ID=${codigo}`;
        window.open(downloadUri);
    }

    public abrirOpcoesLink() {
        const link = this.dadosModeloPop[0].LINKMODELOPOP;
        const linkArquivo = this.dadosModeloPop[0].DSCAMINHOARQ;

        if (link !== null && linkArquivo !== null) {
            this.abrirModalOpcoesAbertura();
        } else if (link !== null) {
            this.abrirLink();
        } else {
            this.abrirPdf();
        }
    }

    public abrirLink() {
        const link = this.dadosModeloPop[0].LINKMODELOPOP;

        if (this.isValidLink(link)) {
            this.openValidLink(link); // Abre o link em nova guia
        }
    }

    /**
     * Valida se um link é válido.
     * @param url - O link a ser validado.
     * @returns Retorna `true` se o link for válido, caso contrário `false`.
     */
    public isValidLink(url: string): boolean {
        try {
            const parsedUrl = new URL(url);
            return parsedUrl.protocol === 'http:' || parsedUrl.protocol === 'https:';
        } catch (error) {
            Utilities.alertMessage('O link inserido é inválido, e, por esse motivo, não foi possível abrir o Modelo POP. Solicite ao supervisor que revise o modelo de procedimento anexado a esta tarefa.', 'error');
            return false;
        }
    }

    /**
     * Abre um link válido em uma nova guia.
     * @param url - O link a ser aberto.
     */
    public openValidLink(url: string): void {
        if (this.isValidLink(url)) {
            window.open(url, '_blank');
        } else {
            Utilities.alertMessage('O link inserido é inválido, e, por esse motivo, não foi possível abrir o Modelo POP. Solicite ao supervisor que revise o modelo de procedimento anexado a esta tarefa.', 'error');
        }
    }

    public montarInformacoesModeloPop() {
        const campoDescricao = Metadata.getInstance<RichText>('descricao-observacao');
        const descricao = this.dadosModeloPop[0].DESCMODELOPOP;
        const observacao = this.dadosModeloPop[0].DSOBSERVACAO;

        const hasDescricao = descricao && descricao !== '<p></p>'; // Verifica se existe descrição válida
        const hasObservacao = observacao !== null; // Verifica se existe observação válida

        campoDescricao.value = null;
        if (hasDescricao && hasObservacao) {
            campoDescricao.value = `${descricao} <br>Informações complementares: ${observacao}`;
        } else if (hasDescricao) {
            campoDescricao.value = descricao;
        } else {
            campoDescricao.value = `Informações complementares: ${observacao}`;
        }
    }

    public montarInformacoesComplementares(nr: any) {
        LoadingService.show();
        const { data } = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas').datasource;
        const observacao = Metadata.getInstance<Textarea>('box-observation');
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-detalhes');
        const botao = Metadata.getInstance<Button>('botao-iniciar-finalizar-tarefa');

        const dados: any = data.find((row: any) => row.NRCHECKLISTAGEND === Number(nr));

        Metadata.getInstance<TextInput>('nrcheck').value = dados.NRCHECKLISTAGEND;
        Metadata.getInstance<TextInput>('idsituacao').value = dados.IDSITUACAOAGEND;
        Metadata.getInstance<TextInput>('tipoMarcacao').value = dados.IDTIPOMARCACAOTAREFA;
        this.nomeTarefa = dados.NMCHECKLISTAGEND;
        this.localTarefa = `${dados.NMLOCALCLIENTE} - ${dados.NMLOCALPAI}`;
        this.dataTarefa = `${dados.DTAGEND} ${dados.HORAAGEND}`;
        this.statusTarefa = this.estilizarStatus(dados.SITUACAO);
        this.nrlocalcli = dados.NRLOCALCLIENTE;

        this.alterarBotao({ component: botao });
        this.carregarEpis(nr);
        this.loadModeloPop(nr);
        this.carregarItens({ component: iterable });

        if (dados.IDOBRIGAOBSERVACAO !== 'S') {
            observacao.clearValidations();
        } else {
            observacao.addValidation('required', {});
        }
        LoadingService.hide();
    }

    public habilitarBotaoSalvarObservacao({ component }: any) {
        if (component.value.trim().replace(' ', '').length > 0) {
            this.desabilitarSalvarObs = false;
        } else {
            this.desabilitarSalvarObs = true;
        }
    }

    public salvarObservacao() {
        if (Utilities.validateForm(Metadata.getInstance<Form>('Form-observation'))) {
            LoadingService.show();
            this.rotaSalvarObservacao().then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    /* Nesta função também contém a tratativa para alterar o status do Item (IDREALIZADO) para 'S'
    tanto no momento da iniciação automática quanto em uma tarefa já em andamento */
    public changeCheckbox({ component }: any) {
        const botao = Metadata.getInstance<Button>('botao-iniciar-finalizar-tarefa');
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;
        const status = Metadata.getInstance<TextInput>('idsituacao').value;
        const iterableCheckbox = Metadata.getInstance<IterableComponentRender>('iterable-cards-detalhes');
        const tipoMarcacao = Metadata.getInstance<TextInput>('tipoMarcacao').value;
        const chkboxS: IDictionary = [];
        const chkboxN: IDictionary = [];

        iterableCheckbox.datasource.data.forEach((itens: IDictionary) => {
            const checkbox = Metadata.getInstance<Checkbox>(`checkbox_${itens.NRTAREFAAGEND}`);

            if (tipoMarcacao === 'U' && checkbox.name !== component.name && checkbox.value === 'S') {
                checkbox.value = 'N';
            }

            if (checkbox.value === 'S') {
                chkboxS.push(itens.NRTAREFAAGEND);
            } else {
                chkboxN.push(itens.NRTAREFAAGEND);
            }
        });

        if (status !== '2') {
            this.iniciarTarefa(nr, 'iniciar', '2');
            this.estilizarBotao({ component: botao }, 'Finalizar');
            botao.disabled = false;
            this.alterarStatusItem(chkboxS, chkboxN, nr);
            this.statusTarefa = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        } else if (status === '2') {
            this.alterarStatusItem(chkboxS, chkboxN, nr);
        }
    }

    public alterarStatusItem(itensMarcados: any, itensNMarcados: any, nrChecklistAgend: any) {
        LoadingService.show();
        this.rotaStatusItem(itensMarcados, itensNMarcados, nrChecklistAgend).then(() => {
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarItens({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;
        this.loadItensTarefas({ component });
        setTimeout(() => {
            Metadata.getInstance<Textarea>('box-observation').value = component.datasource.data[0].DSOBSTAREFAAGEND === null ? '' : component.datasource.data[0].DSOBSTAREFAAGEND;
        }, 1000);
    }

    public estilizarStatus(status: string) {
        if (status === 'Pendente') {
            return "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
        }
        if (status === 'Em andamento') {
            return "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        }
        if (status === 'Atrasado') {
            return "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>A T R A S A D O</span>";
        }
        if (status === 'Concluído') {
            return "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
        }
        return '';
    }

    // Funções da tela de Reportar
    public addTextArea({ component }: IEventParam<Button>) {
        const btOcupado = Metadata.getInstance<Button>('btOcupado');
        const btSemAcesso = Metadata.getInstance<Button>('btSemAcesso');
        const btEspacoLimpo = Metadata.getInstance<Button>('btEspacoLimpo');
        const btOutrosMotivos = Metadata.getInstance<Button>('btOutrosMotivos');
        const textA = Metadata.getInstance<Textarea>('textarea-justificativa');

        btOcupado.outline = !(component.label === btOcupado.label);
        btSemAcesso.outline = !(component.label === btSemAcesso.label);
        btEspacoLimpo.outline = !(component.label === btEspacoLimpo.label);
        btOutrosMotivos.outline = !(component.label === btOutrosMotivos.label);

        if (component.label === btOutrosMotivos.label) {
            textA.isVisible = true;
        } else {
            textA.isVisible = false;
        }
    }

    public abrirReportarBotaoExterno({ component }: IEventParam<Button>) {
        this.visibilidadePaginaInicial = false;
        this.visibilidadeReportar = !this.visibilidadePaginaInicial;
        const nr = component.name.split('_')[1];
        this.nrChecklistAgend = nr;
        this.modoAberturaReportar = true;
    }

    public abrirReportarBotaoInterno() {
        this.visibilidadeDetalhamento = false;
        this.visibilidadeReportar = !this.visibilidadeDetalhamento;
        this.nrChecklistAgend = Metadata.getInstance<TextInput>('nrcheck').value;
        this.modoAberturaReportar = false;
    }

    public limparTelaReportar() {
        Metadata.getInstance<Button>('btOcupado').outline = true;
        Metadata.getInstance<Button>('btSemAcesso').outline = true;
        Metadata.getInstance<Button>('btEspacoLimpo').outline = true;
        Metadata.getInstance<Button>('btOutrosMotivos').outline = true;
        Metadata.getInstance<Textarea>('textarea-justificativa').value = null;
    }

    public fecharPaginaReportar() {
        this.visibilidadeReportar = false;
        if (!this.modoAberturaReportar) {
            this.visibilidadeDetalhamento = true;
        } else {
            this.visibilidadePaginaInicial = true;
        }
        this.limparTelaReportar();
    }

    public reportar() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');
        let justify = ['btOcupado', 'btSemAcesso', 'btEspacoLimpo', 'btOutrosMotivos'].filter((nameButton: string) => Metadata.getInstance<Button>(nameButton).outline === false)[0];

        if (justify === 'btOutrosMotivos') {
            justify = (Metadata.getInstance<Textarea>('textarea-justificativa').value.trim().replace(' ', '').length > 0) ? Metadata.getInstance<Textarea>('textarea-justificativa').value.trim() : justify;
        } else {
            justify = Metadata.getInstance<Button>(justify).label;
        }

        if (justify !== 'btOutrosMotivos') {
            this.rotaReportarTarefa(this.nrChecklistAgend, justify).then(() => {
                Utilities.showSuccessMessage();
                this.loadTarefas({ component: iterable });
                this.visibilidadeReportar = false;
                this.visibilidadePaginaInicial = true;
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Favor preencher o motivo', 'error');
        }
    }

    public async loadTarefas({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = this.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/getMinhasTarefas', filter);
        component.datasource.updateData(response.data.data);
        component.datasource.clearFilter();

        component.datasource.loading = false;
        FiltroUtil.reloadResources(TarefasResources);
    }

    public async loadItensTarefas({ component }: IEventParam<IterableComponentRender>) {
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;
        const filter = [
            { name: 'NRCHECKLISTAGEND', operation: '=', value: nr },
        ];
        const response = await HttpCRUDTranslator.get('/getItensTarefas', filter);
        component.datasource.updateData(response.data.data);

        component.datasource.loading = false;
    }

    public async loadEpisTarefas({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const nr = Metadata.getInstance<TextInput>('nrcheck').value;
        const filter = [
            { name: 'NRCHECKLISTAGEND', operation: '=', value: nr },
        ];

        const response = await HttpCRUDTranslator.get('/getEpisTarefa', filter);
        component.datasource.updateData(response.data.data);
        component.datasource.clearFilter();

        component.datasource.loading = false;
    }

    public async loadModeloPop(nr: any) {
        const filter = [
            { name: 'NRCHECKLISTAGEND', operation: '=', value: nr },
        ];

        const response = await HttpCRUDTranslator.get('/getModeloPopTarefa', filter);

        this.dadosModeloPop = response.data.data;
        this.visibleCardsModeloPop = (this.dadosModeloPop && this.dadosModeloPop.length > 0);
    }

    // ------------------------------ FUNÇÔES GEOLOCALIZAÇÃO ------------------------------
    public areaMapa: any;

    public posAtual: any;

    public parametrizacao: any;

    private google: any;

    private map: any;

    public async googleMaps() {
        const geolocalizacao = await this.buscaGeolocalizacaoLocalAtendimento();
        const loader = new Loader({ apiKey: 'AIzaSyCuKQdQJD_Ae2hKegI7xO47taR2XNassuM', libraries: ['drawing', 'geometry'] });
        this.google = await loader.load();
        this.areaMapa = JSON.parse(geolocalizacao.NRGEOLOCALIZACAO);
        const verifica = this.verificarLocation();
        return verifica;
    }

    // FUNÇÃO PARA O FMAPP VERIFICAR SE OPERADOR ESTÁ NA ÁREA CORRETA PAR INICIAR A TAREFA
    public verificarLocation() {
        const { map } = this.map;
        const novoCirculo = (new this.google.maps.Circle({
            map,
            fillColor: '#ff6961',
            strokeWeight: 2,
            clickable: false,
            editable: false,
            draggable: true,
            radius: parseFloat(this.areaMapa.radius.toFixed(5)),
            center: {
                lat: parseFloat(this.areaMapa.center.lat.toFixed(5)),
                lng: parseFloat(this.areaMapa.center.lng.toFixed(5)),
            },
        })).getBounds();

        return novoCirculo.contains(this.posAtual);
        // return ((this.posAtual.lat - this.areaMapa.center.lat) ** 2) + ((this.posAtual.lng - this.areaMapa.center.lng) ** 2) <= this.areaMapa.radius;
    }

    public async criaMinhaLocalizacao() {
        if (navigator.geolocation) {
            navigator.geolocation.getCurrentPosition((position: any) => {
                const pos = {
                    lat: position.coords.latitude,
                    lng: position.coords.longitude,
                };
                this.posAtual = pos;
            });
        } else {
            Utilities.alertMessage('Favor verificar as permisões do navegador para localização.', 'error');
        }
    }

    public async buscaGeolocalizacaoLocalAtendimento() {
        let resposta: any = '';
        LoadingService.show();
        await HttpCRUDTranslator.post('/getGeolocalizacao', {
            NRLOCALCLIENTE: this.nrlocalcli,
        }).then((Response: any) => {
            resposta = Response;
            // Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
        return resposta.data[0];
    }

    public async getParametrização() {
        await HttpCRUDTranslator.get('/getGeolocalizacaoParametrizacao').then((Response: any) => {
            [this.parametrizacao] = Response.data;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    // FUNÇÕES AUXILIARES PARA CONTROLE DE ROTAS/MIRAGEJS
    public utilizarMirageJs = (window.localStorage.getItem('MIRAGEJS') === 'S');

    public async rotaIniciarTarefa(nr: any, tipo: any, id: any) {
        const metodo = await HttpCRUDTranslator.post('/alterarStatusTarefa', {
            NRCHECKLISTAGEND: nr,
            TIPO: tipo,
            IDSITUACAOAGEND: id,
        });

        return metodo;
    }

    public rotaFinalizarTarefa(nr: any) {
        const metodo = HttpCRUDTranslator.post('/finalizarTarefa', {
            NRCHECKLISTAGEND: nr,
            TIPO: 4,
        });
        return metodo;
    }

    public rotaReportarTarefa(nr: any, justify: any) {
        const metodo = HttpCRUDTranslator.post('/alterarStatusTarefa', {
            NRCHECKLISTAGEND: nr,
            TIPO: 'reportar',
            IDSITUACAOAGEND: 6,
            DSJUSTCANCELAMENTO: justify,
        });
        return metodo;
    }

    public async rotaSalvarObservacao() {
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;
        const ds = Metadata.getInstance<Textarea>('box-observation').value;
        const metodo = HttpCRUDTranslator.post('/salvarObservacao', {
            NRCHECKLISTAGEND: nr,
            DSOBSTAREFAAGEND: ds,
        });
        return metodo;
    }

    public rotaTarefasListadas(component: any) {
        const filter = [
            { name: 'NRCHECKLISTAGEND', operation: '=', value: component.name.split('_')[1] },
            { name: 'TIPO', operation: '=', value: 2 },
        ];
        const metodo = HttpCRUDTranslator.get('/getTarefasListadas', filter);
        return metodo;
    }

    public rotaAnexarImagens(imagensConvertidas: any, nr: any) {
        const metodo = HttpCRUDTranslator.post('/anexarImagensTarefas', {
            IMAGEM: imagensConvertidas,
            NRCHECKLISTAGEND: nr,
        });

        return metodo;
    }

    public async rotaStatusItem(itensMarcados: any, itensNMarcados: any, nrChecklistAgend: any) {
        const metodo = await HttpCRUDTranslator.post('/alterarStatusItem', {
            NRCHECKLISTAGEND: nrChecklistAgend,
            NMARCADOS: itensNMarcados,
            MARCADOS: itensMarcados,
        });

        return metodo;
    }

    public async popularItensTarefas(component: any, nr: any) {
        component.datasource.setFilter({
                NRCHECKLISTAGEND: nr,
        });
    }
}
