import {
    IEventParam,
  Metadata,
} from '@zeedhi/core';
import {
  SelectMultiple, DateRange, Modal, ModalService, Tabs,
} from '@zeedhi/common';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';
import { NaoConformidadesResources } from '@/server/config/Resources';
import { FiltroUtil } from './filtroUtil';

export class Fm00011000NaoConformidadesController {
    private conformidadesGerais!: Modal;

    private conformidadesPrincipais!: Modal;

    public async criarModais() {
        this.conformidadesGerais = await ModalService.createFromJson('fm_00011_000_analise_nao_conformidade/modal/fm_00011_001_filtros_gerais', true, true);
        this.conformidadesPrincipais = await ModalService.createFromJson('fm_00011_000_analise_nao_conformidade/modal/fm_00011_002_filtros_principais', true, true);
    }

    public destruirModais() {
        this.conformidadesGerais?.destroy();
        this.conformidadesPrincipais?.destroy();
    }

    public async fecharModalGerais() {
        this.conformidadesGerais.hide();
    }

    public async fecharModalPrincipais() {
        this.conformidadesPrincipais.hide();
    }

    public abrirModal() {
        const arrayTabs = Metadata.getInstance<Tabs>('tabs-nao-conformidades');

        if (arrayTabs.tabs[arrayTabs.activeTab].name === 'tab-gerais') {
            this.conformidadesGerais.show();
        } else {
            this.conformidadesPrincipais.show();
        }
    }

    public setFilterConformidadesGerais() {
        const card = Metadata.getInstance<IterableComponentRender>('iterable-cards-nao-conformidades-gerais');

        const filtros = {
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-local').value,
            NRTIPOLOCALCLI: Metadata.getInstance<SelectMultiple>('filtroTipoLocal').value,
            CDOPERADOR: Metadata.getInstance<SelectMultiple>('filtroUsuario').value,
            CDOPERADORPAI: Metadata.getInstance<SelectMultiple>('filtroGestor').value,
            // filtroDataRange: Metadata.getInstance<DateRange>('data').value,
        };

        const filtrosFiltrados = FiltroUtil.filtrarFiltros(filtros);

        card.datasource.setFilter(filtrosFiltrados);
        this.loadNaoConformidadesGerais({ component: card });
        this.fecharModalGerais();
    }

    public limparFiltrosGerais() {
        const card = Metadata.getInstance<IterableComponentRender>('iterable-cards-nao-conformidades-gerais');

        Metadata.getInstance<SelectMultiple>('filtro-local').value = null;
        Metadata.getInstance<SelectMultiple>('filtroTipoLocal').value = null;
        Metadata.getInstance<SelectMultiple>('filtroUsuario').value = null;
        Metadata.getInstance<SelectMultiple>('filtroGestor').value = null;
        Metadata.getInstance<DateRange>('data').value = null;

        card.datasource.clearFilter();
        this.loadNaoConformidadesGerais({ component: card });
    }

    public limparFiltroPrincipais() {
        const card = Metadata.getInstance<IterableComponentRender>('iterable-cards-nao-conformidades-principais');

        Metadata.getInstance<DateRange>('filtroDataRange').value = null;
        card.datasource.clearFilter();
        this.loadPrincipaisNaoConformidades({ component: card });
    }

    public setFilterConformidadesPrincipais() {
        const card = Metadata.getInstance<IterableComponentRender>('iterable-cards-nao-conformidades-principais');

        const filtros = {
            filtroDataRange: Metadata.getInstance<DateRange>('filtroDataRange').value,
        };

        const filtrosFiltrados = FiltroUtil.filtrarFiltros(filtros);

        card.datasource.setFilter(filtrosFiltrados);
        this.loadPrincipaisNaoConformidades({ component: card });
        this.fecharModalPrincipais();
    }

    public async loadNaoConformidadesGerais({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = FiltroUtil.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/getNaoConformidadesGerais', filter);
        component.datasource.updateData(response.data.data);
        component.datasource.clearFilter();

        component.datasource.loading = false;
        FiltroUtil.reloadResources(NaoConformidadesResources);
    }

    public async loadPrincipaisNaoConformidades({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = FiltroUtil.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/getPrincipaisNaoConformidades', filter);
        component.datasource.updateData(response.data.data);
        component.datasource.clearFilter();

        component.datasource.loading = false;
    }
}
