import { Utilities } from '@/util/Utilities';
import { Network } from '@capacitor/network';
import { reload } from '@zeedhi/zeedhi-miragejs';

export class FiltroUtil {
    static montarFiltros(valoresFiltro: any) {
        return Object.entries(valoresFiltro ?? {})
            .filter(([, value]) => (Array.isArray(value) ? value.length > 0 : value !== null && value !== undefined))
            .reduce((acc: { name: string; operation: string; value: any[] }[], [name, value]) => {
                const existing = acc.find((filter) => filter.name === name);
                if (existing) {
                    existing.value = [...existing.value, ...(Array.isArray(value) ? value : [value])];
                } else {
                    acc.push({
                        name,
                        operation: '=',
                        value: Array.isArray(value) ? value : [value],
                    });
                }
                return acc;
            }, [])
            .map((filter) => ({
                ...filter,
                value: filter.value.length > 1 ? filter.value : filter.value[0],
            }));
    }

    static filtrarFiltros(filtros: any) {
        return Object.fromEntries(
            Object.entries(filtros).filter(([, value]) => {
                if (Array.isArray(value)) {
                    return value.length > 0;
                }
                return value != null;
            }),
        );
    }

    static async reloadResources(resources: Record<string, any>): Promise<void> {
        try {
            const { connected } = await Network.getStatus();

            if (!connected) {
                return;
            }

            const resourceNames = Object.keys(resources);
            await Promise.all(resourceNames.map(reload));
        } catch (error) {
            Utilities.showDialog('Erro', 'error', 'Erro ao Recarregar Offline');
        }
    }
}
